//
// Copyright Kuzminov Sergey (kuzminov_sergey@mail.ru) 2021
//
// Distributed under the Unlicense License. (See accompanying
// file UNLICENSE.txt or copy at http://unlicense.org/)
//
#ifdef MK_CHART
#include <QApplication>
#else
    #ifdef MK_RTSP
        #include <QGuiApplication>
    #else
        #include <QCoreApplication>
    #endif
#endif

#include <QDebug>
#include <QThread>

#include "WorkerThread.h"

#include "include/AlreadyRunning/AlreadyRunning.h"
#include "include/LogFiles/LogFiles.h"

quint32 applicationVersion;
QString mainPath;
const QDataStream::Version dataStreamVersion = QDataStream::Qt_5_15;

//------------------------------------------------------------------------------
int main(int argc, char *argv[])
{
    qputenv("QT_QPA_PLATFORM", "offscreen");
#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
    qRegisterMetaType <qintptr> ("qintptr");
    qRegisterMetaType <QVector<QPointF>> ("QVector<QPointF>");
#endif

#ifdef Q_OS_WIN
    // Сборки Qt от msys2 не задают путь для поиска плагинов в подкаталоге plugins, исправим это в самой программе.
    //QCoreApplication::addLibraryPath(QLatin1String("plugins"));
#endif

    //------------------------------------------------------------------
    // Название и версия программы
    //------------------------------------------------------------------
#ifdef MK_CHART
    QApplication a(argc, argv);
#else
#ifdef MK_RTSP
    QGuiApplication a(argc, argv);
#else
    QCoreApplication a(argc, argv);
#endif
#endif
    a.setOrganizationName  ( QStringLiteral("www.arm-skif.ru") );
    a.setOrganizationDomain( QStringLiteral("www.arm-skif.ru") );
    a.setApplicationVersion( QStringLiteral("3.0.0") );
    applicationVersion = 0x00030000; // Для передачи по сети

    //------------------------------------------------------------------
    // Путь к программе
    //------------------------------------------------------------------
    mainPath = QCoreApplication::applicationDirPath();

    //------------------------------------------------------------------
    // Настройка кодировки консоли Windows для правильной работы
    // QString::toLocal8Bit() - применяется при выводе в консоль при
    // помощи: qDebug(). qWarning(), qInfo(), qCritical(), qFatal()
    //------------------------------------------------------------------
#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
#ifdef Q_OS_WIN
    if (QLocale::system().country() == QLocale::RussianFederation)
        QTextCodec::setCodecForLocale(QTextCodec::codecForName("IBM 866"));
#else
    if (QLocale::system().country() == QLocale::AnyCountry)
        QTextCodec::setCodecForLocale(QTextCodec::codecForName("UTF-8"));
#endif
#endif

    //------------------------------------------------------------------
    // Для работы с файлами лога
    //------------------------------------------------------------------
    logfiles::init();

    // Включение журналов modbus
    //QLoggingCategory::setFilterRules(QStringLiteral("qt.modbus.lowlevel = true"));
    //QLoggingCategory::setFilterRules(QStringLiteral("qt.modbus = true"));
    //QLoggingCategory::setFilterRules(QStringLiteral("qt.modbus* = true"));

    //------------------------------------------------------------------
    // Файл перевода
    //------------------------------------------------------------------
#ifdef Embedded
    const QLocale locale = QLocale(QLocale::Russian, QLocale::Russia);
    QLocale::setDefault(locale);

#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
    QString translationsPath = QStringLiteral("/usr/share/qt5/translations");
#else
    QString translationsPath = QStringLiteral("/usr/share/qt6/translations");
#endif

    //Варианты: qtbase, qtserialport, qtscript, qtwebsockets, qtmultimedia
    const QStringList listTranslator = {QStringLiteral("qtbase"), QStringLiteral("qtserialport"), QStringLiteral("qtscript")};

    for (const QString &strTranslator : listTranslator){
        QTranslator *translator = new QTranslator(qApp);
        if (translator->load(locale, strTranslator, QStringLiteral("_"), translationsPath))
            qApp->installTranslator(translator);
    }

#endif

    //------------------------------------------------------------------
    // Определяем запущено ли уже приложение
    //------------------------------------------------------------------
    AlreadyRunning alreadyRunning;
    if (alreadyRunning.isRunning()) {
        qCritical("Попытка запустить вторую копию программы");
        return -1;
    }

    //------------------------------------------------------------------
    // Запустить всю логику программы в отдельном потоке, т.к.
    // главный поток будет использоваться для операций с графикой,
    // которую нельзя переместить в другой поток из-за ограничений Qt.
    //------------------------------------------------------------------
    WorkerThread *workerThread = new WorkerThread(&a);
    workerThread->start();
    QObject::connect(&a, &QCoreApplication::aboutToQuit, &a, [=]
    {
        workerThread->quit();
        workerThread->wait(300);
    });

    return a.exec();
}
